// decodable-nnet-online-for-hark.cc
// If you want to put this file under the directory, it file is placed in under the "nnet" directory.

#ifdef _MSC_VER
#define NOMINMAX
#endif
#include "decodable-nnet1-online-for-hark.h"

//#include "nnet/online-nnet-decodable.h"
#include "nnet3/nnet-utils.h"

namespace kaldi {
namespace nnet1 {

DecodableNnetOnlineForHark::DecodableNnetOnlineForHark(
        const kaldi::nnet1::Nnet &nnet,
        kaldi::nnet1::PdfPrior &pdf_prior,
        const kaldi::TransitionModel &trans_model,
        const DecodableNnetOnlineForHarkOptions &opts,
        //kaldi::OnlineFeatureInterface *features_if):
        std::shared_ptr<kaldi::OnlineFeatureInterface> features_if):
    features_if_(features_if),
    nnet_(nnet),
    pdf_prior_(pdf_prior),
    trans_model_(trans_model),
    opts_(opts),
    feat_dim_(features_if->Dim()),
    num_pdfs_(nnet.OutputDim()),
    begin_frame_(-1) {
  KALDI_ASSERT(opts_.max_nnet_batch_size > 0);
}

kaldi::BaseFloat DecodableNnetOnlineForHark::LogLikelihood(kaldi::int32 frame, kaldi::int32 index) {
  ComputeForFrame(frame);
  kaldi::int32 pdf_id = trans_model_.TransitionIdToPdf(index);
  KALDI_ASSERT(frame >= begin_frame_ && (frame < begin_frame_ + scaled_loglikes_.NumRows()));
  return scaled_loglikes_(frame - begin_frame_, pdf_id);
}

bool DecodableNnetOnlineForHark::IsLastFrame(kaldi::int32 frame) const {
  return features_if_->IsLastFrame(frame);
}

kaldi::int32 DecodableNnetOnlineForHark::NumFramesReady() const {
  return features_if_->NumFramesReady();
}

void DecodableNnetOnlineForHark::ComputeForFrame(kaldi::int32 frame) {
  kaldi::int32 features_ready = features_if_->NumFramesReady();
  //bool input_finished = features_if_->IsLastFrame(features_ready - 1);
  //KALDI_LOG << features_ready;
  if(frame >= begin_frame_ && (frame < begin_frame_ + scaled_loglikes_.NumRows())){
    return;
  }

  kaldi::int32 input_frame_begin = frame;
  kaldi::int32 max_possible_input_frame_end = features_ready;
  kaldi::int32 input_frame_end =
      std::min<kaldi::int32>(max_possible_input_frame_end,
                             input_frame_begin + opts_.max_nnet_batch_size);

  KALDI_ASSERT(input_frame_end > input_frame_begin);
  kaldi::Matrix<kaldi::BaseFloat> features(input_frame_end - input_frame_begin, feat_dim_);
  for(kaldi::int32 t = input_frame_begin; t < input_frame_end; t++){
    kaldi::SubVector<kaldi::BaseFloat> row(features, t - input_frame_begin);
    features_if_->GetFrame(t, &row);
  }
  kaldi::CuMatrix<kaldi::BaseFloat> cu_features;
  cu_features.Swap(&features);

  kaldi::int32 num_frames_out = input_frame_end - input_frame_begin;
  kaldi::CuMatrix<kaldi::BaseFloat> cu_posteriors(num_frames_out, num_pdfs_);

  nnet_.Feedforward(cu_features, &cu_posteriors);
  pdf_prior_.SubtractOnLogpost(&cu_posteriors);
  cu_posteriors.Scale(opts_.acoustic_scale);

  scaled_loglikes_.Resize(0, 0);
  cu_posteriors.Swap(&scaled_loglikes_);

  begin_frame_ = frame;
}

} // namespace nnet1
} // namespace kaldi
