// decodable-nnet3-online-for-hark.h
// Implements like "nnet3/decodable-simple-looped.h".
// Implements like "nnet3/decodable-online-looped.h".

#ifndef KALDI_DECODABLE_NNET3_ONLINE_FOR_HARK_H_
#define KALDI_DECODABLE_NNET3_ONLINE_FOR_HARK_H_

#include "itf/online-feature-itf.h"
#include "itf/decodable-itf.h"
#include "hmm/transition-model.h"
// nnet3
#include "nnet3/am-nnet-simple.h"
#include "nnet3/nnet-compute.h"
#include "nnet3/nnet-optimize.h"
#include "nnet3/nnet-compile-looped.h"
#include "nnet3/decodable-simple-looped.h"
#include "nnet3/nnet-utils.h"

namespace kaldi {
namespace nnet3 {

class DecodableNnet3SimpleOnlineForHark: public DecodableInterface {
 public:
  DecodableNnet3SimpleOnlineForHark(
                                    const DecodableNnetSimpleLoopedInfo &decodable_info,
                                    const kaldi::TransitionModel &trans_model,
                                    const NnetSimpleLoopedComputationOptions &opts,
                                    kaldi::OnlineFeatureInterface *features_if,
                                    kaldi::OnlineFeatureInterface *ivector_if = nullptr);

  ~DecodableNnet3SimpleOnlineForHark(){ }

  /// Returns the scaled log likelihood
  virtual kaldi::BaseFloat LogLikelihood(kaldi::int32 subsampled_frame, kaldi::int32 index);

  virtual bool IsLastFrame(kaldi::int32 subsampled_frame) const;

  virtual kaldi::int32 NumFramesReady() const;

  /// Indices are one-based!  This is for compatibility with OpenFst.
  virtual kaldi::int32 NumIndices() const { return trans_model_.NumTransitionIds(); }

  kaldi::int32 FrameSubsamplingFactor() const { return opts_.frame_subsampling_factor; }

 private:
  /// If the neural-network outputs for this frame are not cached, this function
  /// computes them (and possibly also some later frames).  Note:
  /// the frame-index is called 'subsampled_frame' because if frame-subsampling-factor
  /// is not 1, it's an index that is "after subsampling", i.e. it changes more
  /// slowly than the input-feature index.
  inline void EnsureFrameIsComputed(int32 subsampled_frame){
    KALDI_ASSERT(subsampled_frame >= current_log_post_subsampled_offset_ && "Frames must be accessed in order.");
    while(subsampled_frame >= current_log_post_subsampled_offset_ + current_log_post_.NumRows()){
      AdvanceChunk();
    }
  }

  // This function does the computation for the next chunk.  It will change
  // current_log_post_ and current_log_post_subsampled_offset_, and
  // increment num_chunks_computed_.
  void AdvanceChunk();

  kaldi::OnlineFeatureInterface *features_if_;
  kaldi::OnlineFeatureInterface *ivector_if_;
  const kaldi::TransitionModel &trans_model_;
  const DecodableNnetSimpleLoopedInfo decodable_info_;
  NnetSimpleLoopedComputationOptions opts_;

  kaldi::Matrix<kaldi::BaseFloat> current_log_post_;
  kaldi::int32 num_chunks_computed_;
  kaldi::int32 current_log_post_subsampled_offset_;

  kaldi::nnet3::NnetComputer computer_;

  //KALDI_DISALLOW_COPY_AND_ASSIGN(DecodableNnet3SimpleOnlineForHark);
};

} // namespace nnet3
} // namespace kaldi

#endif // KALDI_DECODABLE_NNET3_ONLINE_FOR_HARK_H_
