
#ifndef MP_RESULT_SERVER_H_
#define MP_RESULT_SERVER_H_

#include "hark-support.h"
//DEBUG//#include "stdio_synchronizer.hpp"
#include "uuid_generator.hpp"
#include "io_manager.hpp"
#include "tcp_server.hpp"
#include "tcp_socket.hpp"
#include "thread_pool.hpp"
#include "wsa_wrapper.hpp"

//DEBUG//#include <sstream>
#include <iostream>
#include <mutex>
//#include <map>
#include <unordered_map>
#include <deque>
#include <utility>
#include <functional>
#include <cstdint>

class result_server {
public:
  result_server(){
  }

  ~result_server(){
    std::lock_guard<std::mutex> lock(m_buffers_mutex);
    buffers.erase(buffers.begin(), buffers.end());
  }

  result_server(const std::string& host, std::uint32_t port){
    s.start(host, port, [&](const std::shared_ptr<multiplatform::tcp_client>& client) -> bool {
      //DEBUG// {
      //DEBUG//   std::stringstream ss;
      //DEBUG//   ss << "Result receiver client connected: " << client->get_host() << ":" << client->get_port() << " unique_key=" << client->get_uuid();
      //DEBUG//   _raw_print_with_endl(ss.str());
      //DEBUG// }
      {
        std::lock_guard<std::mutex> lock(m_buffers_mutex);
        buffers[client->get_uuid()] = client;
      }
      client->async_read({1024, std::bind(&result_server::on_new_message, this, client, std::placeholders::_1)});
      return true;
    });
  }

  std::size_t broadcast(std::string text){
    std::vector<char> vec(text.begin(), text.end());
    if(vec.size()>0){
      std::lock_guard<std::mutex> lock(m_buffers_mutex);
      for(auto buffer : buffers){
        if(buffer.second->is_connected()){
          buffer.second->async_write({vec, nullptr});
        }
      }
    }
    return 0;
  }

  bool add_recogend_key(std::string& key){
    bool ret = false;
    {
      std::lock_guard<std::mutex> lock(m_emap_mutex);
      auto p = emap.insert(std::pair<std::string, bool>(key, true));
      ret = p.second;
    }
    return ret;
  }

  bool erase_recogend_key(std::string& key){
    bool ret = false;
    {
      std::lock_guard<std::mutex> lock(m_emap_mutex);
      if(emap.empty()){ return false; }
      if(emap.erase(key) != 0){ ret = true; }
    }
    return ret;
  }

  bool find_recogend_key(std::string& key){
    bool ret = false;
    {
      std::lock_guard<std::mutex> lock(m_emap_mutex);
      if(emap.empty()){ return false; }
      if(emap.find(key) != emap.end()){ ret = true; }
    }
    return ret;
  }

  void on_new_message(const std::shared_ptr<multiplatform::tcp_client>& client, multiplatform::tcp_client::read_response& res) {
    if(res.success){
      //DEBUG// raw_print_with_endl("Client recv data");
      client->async_read({1024, std::bind(&result_server::on_new_message, this, client, std::placeholders::_1)});
    }
    else {
      //DEBUG// raw_print_with_endl("Client disconnected");
      //client->disconnect();
      {
        std::lock_guard<std::mutex> lock(m_buffers_mutex);
        buffers.erase(client->get_uuid());
      }
      client->disconnect();
    }
  }

  multiplatform::tcp_server s;
  std::unordered_map<std::string, std::shared_ptr<multiplatform::tcp_client> > buffers; // key:uuid, data:buffer
  std::unordered_map<std::string, bool> emap; // recognition end list (recongnition complete sources)
  std::mutex m_buffers_mutex;
  std::mutex m_emap_mutex;
};

#endif // MP_RESULT_SERVER_H_

