// recognizer.h

#ifndef KALDI_RECOGNIZER_H_
#define KALDI_RECOGNIZER_H_

#include <memory>

#include "online-mfcnet.h"
#include "mp/result_server.hpp"

#include <iomanip>
#include <string>
#include <vector>

namespace kaldi {

class Recognizer {
 public:
  Recognizer() {}
  virtual ~Recognizer() {}

  virtual std::shared_ptr<OnlineFeatureInterface> GetOnlineFeatureInterface()=0;
  virtual void Recognize()=0;
  virtual void WriteResult()=0;
  virtual void PrintResult(std::ostream& out)=0;
};

inline std::vector<std::string> string_tokenizer(const std::string& str, const std::string& delimiters)
{
  std::string::size_type start, end, i;
  std::vector<std::string> tokens;
  if(delimiters.empty()){ tokens.push_back(str); return tokens; } // not set delimiter
  i = start = str.find_first_not_of(delimiters);
  end = str.find_first_of(delimiters, start);
  if(start == std::string::npos && end == std::string::npos){
    i = str.size() + 1;
    while(i > 0){ tokens.push_back(""); --i; } // while push null token(s)
    return tokens;
  } // delimiter only
  ////std::cout << "start: " << start << ", end: " << end << std::endl;
  while(i > 0){ tokens.push_back(""); --i; } // while push prefix null token(s)
  //if(start != std::string::npos && end == 1){ i = str.size(); while(i > 0){ tokens.push_back(""); --i; } }
  while(start != std::string::npos){
    tokens.push_back(str.substr(start, (end == std::string::npos) ? std::string::npos : end - start));
    start = str.find_first_not_of(delimiters, end);
    ////std::cout << "start: " << start << ", end: " << end << std::endl;
    if(end != std::string::npos){
      i = (start == std::string::npos ? str.size() : start - 1) - end;
      while(i > 0){ tokens.push_back(""); --i; }
    }
    end = str.find_first_of(delimiters, start);
  }
  return tokens;
}

} // namespace kaldi

#endif
